/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: owriteablestream.hxx,v $
 *
 *  $Revision: 1.14 $
 *
 *  last change: $Author: obo $ $Date: 2006/03/24 13:21:09 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef _WRITESTREAM_HXX_
#define _WRITESTREAM_HXX_

#ifndef _COM_SUN_STAR_IO_XINPUTSTREAM_HPP_
#include <com/sun/star/io/XInputStream.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XOUTPUTSTREAM_HPP_
#include <com/sun/star/io/XOutputStream.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XSTREAM_HPP_
#include <com/sun/star/io/XStream.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XSEEKABLE_HPP_
#include <com/sun/star/io/XSeekable.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XTRUNCATE_HPP_
#include <com/sun/star/io/XTruncate.hpp>
#endif

#ifndef _COM_SUN_STAR_PACKAGES_XDATASINKENCRSUPPORT_HPP_
#include <com/sun/star/packages/XDataSinkEncrSupport.hpp>
#endif
#ifndef _COM_SUN_STAR_PACKAGES_NOENCRYPTIONEXCEPTION_HPP_
#include <com/sun/star/packages/NoEncryptionException.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XEVENTLISTENER_HPP_
#include <com/sun/star/lang/XEventListener.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XSINGLESERVICEFACTORY_HPP_
#include <com/sun/star/lang/XSingleServiceFactory.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XCOMPONENT_HPP_
#include <com/sun/star/lang/XComponent.hpp>
#endif

#ifndef _COM_SUN_STAR_EMBED_XENCRYPTIONPROTECTEDSOURCE_HPP_
#include <com/sun/star/embed/XEncryptionProtectedSource.hpp>
#endif

#ifndef _COM_SUN_STAR_CONTAINER_XNAMECONTAINER_HPP_
#include <com/sun/star/container/XNameContainer.hpp>
#endif


#ifndef _CPPUHELPER_IMPLBASE1_HXX_
#include <cppuhelper/implbase1.hxx>
#endif

#ifndef _CPPUHELPER_IMPLBASE8_HXX_
#include <cppuhelper/implbase8.hxx>
#endif

#ifndef _CPPUHELPER_INTERFACECONTAINER_H_
#include <cppuhelper/interfacecontainer.h>
#endif

#include <list>

#include "ocompinstream.hxx"
#include "mutexholder.hxx"


struct PreCreationStruct
{
	SotMutexHolderRef m_rMutexRef;

	PreCreationStruct()
	: m_rMutexRef( new SotMutexHolder )
	{}

};

struct WSInternalData_Impl
{
	SotMutexHolderRef m_rSharedMutexRef;
	::cppu::OInterfaceContainerHelper m_aListenersContainer; // list of listeners
	
	// the mutex reference MUST NOT be empty
	WSInternalData_Impl( const SotMutexHolderRef rMutexRef )
	: m_rSharedMutexRef( rMutexRef )
	, m_aListenersContainer( rMutexRef->GetMutex() )
	{}
};

typedef ::std::list< OInputCompStream* > InputStreamsList_Impl;

struct OStorage_Impl;
class OWriteStream;

struct OWriteStream_Impl : public PreCreationStruct
{
	friend struct OStorage_Impl;
	friend class OWriteStream;
	friend class OInputCompStream;

	OWriteStream*	m_pAntiImpl;
	::rtl::OUString m_aTempURL;
	
	InputStreamsList_Impl m_aInputStreamsList;

    sal_Bool                        m_bHasDataToFlush;    // only modified elements will be sent to the original content
    sal_Bool                        m_bFlushed;      // sending the streams is coordinated by the root storage of the package

	::com::sun::star::uno::Reference< ::com::sun::star::packages::XDataSinkEncrSupport > m_xPackageStream;

	::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > m_xFactory;

	OStorage_Impl* m_pParent;

	::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue > m_aProps;

	sal_Bool m_bForceEncrypted;

	sal_Bool m_bUseCommonPass;
	sal_Bool m_bHasCachedPassword;
	::rtl::OUString m_aPass;

	::com::sun::star::uno::Reference< ::com::sun::star::lang::XSingleServiceFactory > m_xPackage;

	sal_Bool m_bHasInsertedStreamOptimization;

private:
	::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > GetServiceFactory();

	::rtl::OUString GetFilledTempFile();
	::com::sun::star::uno::Reference< ::com::sun::star::io::XStream >		GetTempFileAsStream();
	::com::sun::star::uno::Reference< ::com::sun::star::io::XInputStream >	GetTempFileAsInputStream();

	void CopyTempFileToOutput( ::com::sun::star::uno::Reference< ::com::sun::star::io::XOutputStream > xOutStream );

	::com::sun::star::uno::Reference< ::com::sun::star::io::XStream > GetStream_Impl( sal_Int32 nStreamMode );

	::rtl::OUString GetCommonRootPass() throw ( ::com::sun::star::packages::NoEncryptionException );

	::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue > ReadPackageStreamProperties();
	::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue > InsertOwnProps(
							const ::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue >& aProps,
							sal_Bool bUseCommonPass );

public:
	OWriteStream_Impl( OStorage_Impl* pParent,
					   ::com::sun::star::uno::Reference< ::com::sun::star::packages::XDataSinkEncrSupport > xPackageStream,
					   ::com::sun::star::uno::Reference< ::com::sun::star::lang::XSingleServiceFactory > xPackage,
					   ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > xFactory,
					   sal_Bool bForceEncrypted );

	~OWriteStream_Impl();

	sal_Bool UsesCommonPass_Impl() { return m_bUseCommonPass; }
	sal_Bool HasTempFile_Impl() { return ( m_aTempURL.getLength() != 0 ); }

	sal_Bool HasWriteOwner_Impl() { return ( m_pAntiImpl != NULL ); }

	void InsertIntoPackageFolder(
			const ::rtl::OUString& aName,
			const ::com::sun::star::uno::Reference< ::com::sun::star::container::XNameContainer >& xParentPackageFolder );

	void SetToBeCommited() { m_bFlushed = sal_True; }

	sal_Bool HasCachedPassword() { return m_bHasCachedPassword; }
	::rtl::OUString GetCachedPassword() { return m_aPass; }
	sal_Bool IsModified() { return m_bHasDataToFlush || m_bFlushed; }

	sal_Bool IsEncrypted();
	void SetDecrypted();
	void SetEncryptedWithPass( const ::rtl::OUString& aPass );

	void DisposeWrappers();

	void InsertStreamDirectly(
			const ::com::sun::star::uno::Reference< ::com::sun::star::io::XInputStream >& xInStream,
			const ::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue >& aProps );

	void Commit();
	void Revert();

	void Free( sal_Bool bMust ); // allows to try to disconnect from the temporary stream
				 				// in case bMust is set to sal_True the method 
								// will throw exception in case the file is still busy

	void SetModified(); // can be done only by parent storage after renaming

	::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue > GetStreamProperties();

	void CopyInternallyTo_Impl( const ::com::sun::star::uno::Reference< ::com::sun::star::io::XStream >& xDestStream,
								const ::rtl::OUString& aPass );
	void CopyInternallyTo_Impl( const ::com::sun::star::uno::Reference< ::com::sun::star::io::XStream >& xDestStream );

	::com::sun::star::uno::Reference< ::com::sun::star::io::XStream > GetStream(
						sal_Int32 nStreamMode,
						const ::rtl::OUString& aPass );

	::com::sun::star::uno::Reference< ::com::sun::star::io::XStream > GetStream(
						sal_Int32 nStreamMode );


	::com::sun::star::uno::Reference< ::com::sun::star::io::XInputStream > GetRawInStream();
	::com::sun::star::uno::Reference< ::com::sun::star::io::XInputStream > GetPlainRawInStream();

	void InputStreamDisposed( OInputCompStream* pStream );

	void CreateReadonlyCopyBasedOnData(
					const ::com::sun::star::uno::Reference< ::com::sun::star::io::XInputStream >& xDataToCopy,
					const ::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue >& aProps,
					sal_Bool bUseCommonPass,
					::com::sun::star::uno::Reference< ::com::sun::star::io::XStream >& xTargetStream );

	void GetCopyOfLastCommit( ::com::sun::star::uno::Reference< ::com::sun::star::io::XStream >& xTargetStream );
	void GetCopyOfLastCommit(
				  ::com::sun::star::uno::Reference< ::com::sun::star::io::XStream >& xTargetStream,
							const ::rtl::OUString& aPass );
};

class OWriteStream : public cppu::WeakImplHelper8 < ::com::sun::star::io::XInputStream
													,::com::sun::star::io::XOutputStream
													,::com::sun::star::io::XStream
													,::com::sun::star::io::XSeekable
													,::com::sun::star::io::XTruncate
													,::com::sun::star::lang::XComponent
													,::com::sun::star::embed::XEncryptionProtectedSource
													,::com::sun::star::beans::XPropertySet >
{
	friend struct OWriteStream_Impl;

protected:
	::com::sun::star::uno::Reference < ::com::sun::star::io::XInputStream > m_xInStream;
	::com::sun::star::uno::Reference < ::com::sun::star::io::XOutputStream > m_xOutStream;
	::com::sun::star::uno::Reference < ::com::sun::star::io::XSeekable > m_xSeekable;

	OWriteStream_Impl* m_pImpl;
	WSInternalData_Impl* m_pData;

	sal_Bool m_bInStreamDisconnected;
	sal_Bool m_bInitOnDemand;

	OWriteStream( OWriteStream_Impl* pImpl );
	OWriteStream( OWriteStream_Impl* pImpl, ::com::sun::star::uno::Reference< ::com::sun::star::io::XStream > xStream );

	void CloseOutput_Impl();

	void CopyToStreamInternally_Impl( const ::com::sun::star::uno::Reference< ::com::sun::star::io::XStream >& xStream );

	void ModifyParentUnlockMutex_Impl( ::osl::ResettableMutexGuard& aGuard );

public:

	virtual ~OWriteStream();

	void CheckInitOnDemand();

	// XInputStream
    virtual sal_Int32 SAL_CALL readBytes( ::com::sun::star::uno::Sequence< sal_Int8 >& aData, sal_Int32 nBytesToRead ) 
		throw(::com::sun::star::io::NotConnectedException, ::com::sun::star::io::BufferSizeExceededException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual sal_Int32 SAL_CALL readSomeBytes( ::com::sun::star::uno::Sequence< sal_Int8 >& aData, sal_Int32 nMaxBytesToRead ) throw(::com::sun::star::io::NotConnectedException, ::com::sun::star::io::BufferSizeExceededException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL skipBytes( sal_Int32 nBytesToSkip ) 
		throw(::com::sun::star::io::NotConnectedException, ::com::sun::star::io::BufferSizeExceededException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual sal_Int32 SAL_CALL available(  ) 
		throw(::com::sun::star::io::NotConnectedException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL closeInput(  ) 
		throw(::com::sun::star::io::NotConnectedException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);

	// XOutputStream
    virtual void SAL_CALL writeBytes( const ::com::sun::star::uno::Sequence< sal_Int8 >& aData ) throw (::com::sun::star::io::NotConnectedException, ::com::sun::star::io::BufferSizeExceededException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL flush(  ) throw (::com::sun::star::io::NotConnectedException, ::com::sun::star::io::BufferSizeExceededException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL closeOutput(  ) throw (::com::sun::star::io::NotConnectedException, ::com::sun::star::io::BufferSizeExceededException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);

	//XSeekable
    virtual void SAL_CALL seek( sal_Int64 location ) throw (::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual sal_Int64 SAL_CALL getPosition() throw (::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);
    virtual sal_Int64 SAL_CALL getLength() throw (::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);

	//XStream
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::io::XInputStream > SAL_CALL getInputStream(  ) throw (::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::io::XOutputStream > SAL_CALL getOutputStream(  ) throw (::com::sun::star::uno::RuntimeException);

	// XTruncate
    virtual void SAL_CALL truncate() throw (::com::sun::star::io::IOException, ::com::sun::star::uno::RuntimeException);

	//XComponent
    virtual void SAL_CALL dispose() throw (::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL addEventListener( const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener >& xListener ) throw (::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL removeEventListener( const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener >& aListener ) throw (::com::sun::star::uno::RuntimeException);

	//XEncryptionProtectedSource
    virtual void SAL_CALL setEncryptionPassword( const ::rtl::OUString& aPass )
		throw ( ::com::sun::star::uno::RuntimeException,
				::com::sun::star::io::IOException );
    virtual void SAL_CALL removeEncryption()
		throw ( ::com::sun::star::uno::RuntimeException,
				::com::sun::star::io::IOException );

	//XPropertySet
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySetInfo > SAL_CALL getPropertySetInfo() throw ( ::com::sun::star::uno::RuntimeException );
    virtual void SAL_CALL setPropertyValue( const ::rtl::OUString& aPropertyName, const ::com::sun::star::uno::Any& aValue ) throw ( ::com::sun::star::beans::UnknownPropertyException, ::com::sun::star::beans::PropertyVetoException, ::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException );
    virtual ::com::sun::star::uno::Any SAL_CALL getPropertyValue( const ::rtl::OUString& PropertyName ) throw ( ::com::sun::star::beans::UnknownPropertyException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException );
    virtual void SAL_CALL addPropertyChangeListener( const ::rtl::OUString& aPropertyName, const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertyChangeListener >& xListener ) throw ( ::com::sun::star::beans::UnknownPropertyException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException );
    virtual void SAL_CALL removePropertyChangeListener( const ::rtl::OUString& aPropertyName, const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertyChangeListener >& aListener ) throw ( ::com::sun::star::beans::UnknownPropertyException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException );
    virtual void SAL_CALL addVetoableChangeListener( const ::rtl::OUString& PropertyName, const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XVetoableChangeListener >& aListener ) throw ( ::com::sun::star::beans::UnknownPropertyException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException );
    virtual void SAL_CALL removeVetoableChangeListener( const ::rtl::OUString& PropertyName, const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XVetoableChangeListener >& aListener ) throw ( ::com::sun::star::beans::UnknownPropertyException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException );

};

#endif

