/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: filenotation.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: rt $ $Date: 2005/09/08 21:04:43 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef _SVX_FILENOTATION_HXX_
#include "filenotation.hxx"
#endif
#ifndef _OSL_FILE_H_
#include <osl/file.h>
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#ifndef _URLOBJ_HXX 
#include <tools/urlobj.hxx>
#endif

//.........................................................................
namespace svx
{
//.........................................................................

	//=====================================================================
	//= OFileNotation
	//=====================================================================
	//---------------------------------------------------------------------
	OFileNotation::OFileNotation( const ::rtl::OUString& _rFileOrPath )
	{
		construct( _rFileOrPath );
	}

	//---------------------------------------------------------------------
	OFileNotation::OFileNotation( const ::rtl::OUString& _rFileOrPath, NOTATION _eInputNotation )
	{
		// _eInputNotation is obsolete ... sinve we support hierarchy and wfs URLs, too, it would be way too much
		// to correct all callers, We simply detect the format every time ...
		construct( _rFileOrPath );
	}

	//---------------------------------------------------------------------
	void OFileNotation::construct( const ::rtl::OUString& _rFileOrPath )
	{
		sal_Bool bSuccess = sal_False;
		// URL notation?
		INetURLObject aParser( _rFileOrPath );
		switch ( aParser.GetProtocol() )
		{
			case INET_PROT_FILE:
				// file URL
				m_sFileURL = _rFileOrPath;
				osl_getSystemPathFromFileURL(m_sFileURL.pData, &m_sSystem.pData);
				bSuccess = sal_True;
				break;

			case INET_PROT_NOT_VALID:
				// assume system notation
				m_sSystem = _rFileOrPath;
				if ((osl_File_E_None != osl_getFileURLFromSystemPath(m_sSystem.pData, &m_sFileURL.pData)) && (0 == m_sFileURL.getLength()))
				{
					if (_rFileOrPath.getLength())
					{
						INetURLObject aSmartParser;
						aSmartParser.SetSmartProtocol(INET_PROT_FILE);
						if (aSmartParser.SetSmartURL(_rFileOrPath))
						{
							m_sFileURL = aSmartParser.GetMainURL(INetURLObject::NO_DECODE);
							osl_getSystemPathFromFileURL(m_sFileURL.pData, &m_sSystem.pData);
							bSuccess = sal_True;
						}
					}
				}
				else
					bSuccess = sal_True;
				break;

			default:
				// it's a known scheme, but no file-URL -> assume that bothe the URL representation and the
				// system representation are the URL itself
				m_sSystem = m_sFileURL = _rFileOrPath;
				bSuccess = sal_True;
				break;
		}

		OSL_ENSURE( bSuccess, "OFileNotation::OFileNotation: could not detect the format!" );
	}

	//---------------------------------------------------------------------
	::rtl::OUString OFileNotation::get(NOTATION _eOutputNotation)
	{
		switch (_eOutputNotation)
		{
			case N_SYSTEM: return m_sSystem;
			case N_URL: return m_sFileURL;
		}

		OSL_ENSURE(sal_False, "OFileNotation::get: inavlid enum value!");
		return ::rtl::OUString();
	}

//.........................................................................
}	// namespace svx
//.........................................................................

