/*
 * NameProvider.java
 *
 * Created on 16. M?rz 2003, 12:48
 */
package com.sun.star.wizards.tutorial.executer.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Provides names for several accessibility constants groups.
 *
 * @author $author$
 * @version $Revision: 1.3 $
 */
public class NameProvider {
    private static String[] maStateNames = { "INVALID", // 0
        "ACTIVE", "ARMED", "BUSY", "CHECKED", "COLLAPSED", "DEFUNC", "EDITABLE", "ENABLED", "EXPANDABLE", "EXPANDED", // 10
        "FOCUSABLE", "FOCUSED", "HORIZONTAL", "ICONIFIED", "MODAL", "MULTILINE", "MULTISELECTABLE", "OPAQUE", "PRESSED", "RESIZABLE", // 20
        "SELECTABLE", "SELECTED", "SENSITIVE", "SHOWING", "SINGLE_LINE", "STALE", "TRANSIENT", "VERTICAL", "VISIBLE", "MANAGES_DESCENDANT" // 30
    };
    static final String[] maEventNames = { "[UNKNOWN]", // 0
        "ACTION", "ACTIVE_DESCENDANT", "CARET", "CHILD", "DESCRIPTION", "HYPERTEXT_OFFSET", "NAME", "SELECTION", "STATE", "TABLE_CAPTION_CHANGED", // 10
        "TABLE_COLUMN_DESCRIPTION_CHANGED", "TABLE_COLUMN_HEADER_CHANGED", "TABLE_MODEL_CHANGED", "TABLE_ROW_DESCRIPTION_CHANGED", "TABLE_ROW_HEADER_CHANGED", "TABLE_SUMMARY_CHANGED", "TEXT", "VALUE", "VISIBLE_DATA", "CONTROLLED_BY_PROPERTY", // 20
        "CONTROLLER_FOR_PROPERTY", "LABEL_FOR_PROPERTY", "LABELED_BY_PROPERTY", "MEMBER_OF_PROPERTY", "CONTENT_FLOWS_FROM", "CONTENT_FLOWS_TO", "ALL_CHILDREN_CHANGED", "BOUNDRECT" // 28
    };
    private static HashMap maRoleMap = new HashMap();

    static {
        maRoleMap.put(new Integer(0), "UNKNOWN");
        maRoleMap.put(new Integer(1), "ALERT");
        maRoleMap.put(new Integer(2), "COLUMN_HEADER");
        maRoleMap.put(new Integer(3), "CANVAS");
        maRoleMap.put(new Integer(4), "CHECK_BOX");
        maRoleMap.put(new Integer(5), "CHECK_MENU_ITEM");
        maRoleMap.put(new Integer(6), "COLOR_CHOOSER");
        maRoleMap.put(new Integer(7), "COMBO_BOX");
        maRoleMap.put(new Integer(8), "DATE_EDITOR");
        maRoleMap.put(new Integer(9), "DESKTOP_ICON");
        maRoleMap.put(new Integer(10), "DESKTOP_PANE");
        maRoleMap.put(new Integer(11), "DIRECTORY_PANE");
        maRoleMap.put(new Integer(12), "DIALOG");
        maRoleMap.put(new Integer(13), "DOCUMENT");
        maRoleMap.put(new Integer(14), "EMBEDDED_OBJECT");
        maRoleMap.put(new Integer(15), "END_NOTE");
        maRoleMap.put(new Integer(16), "FILE_CHOOSER");
        maRoleMap.put(new Integer(17), "FILLER");
        maRoleMap.put(new Integer(18), "FONT_CHOOSER");
        maRoleMap.put(new Integer(19), "FOOTER");
        maRoleMap.put(new Integer(20), "FOOTNOTE");
        maRoleMap.put(new Integer(21), "FRAME");
        maRoleMap.put(new Integer(22), "GLASS_PANE");
        maRoleMap.put(new Integer(23), "GRAPHIC");
        maRoleMap.put(new Integer(24), "GROUP_BOX");
        maRoleMap.put(new Integer(25), "HEADER");
        maRoleMap.put(new Integer(26), "HEADING");
        maRoleMap.put(new Integer(27), "HYPER_LINK");
        maRoleMap.put(new Integer(28), "ICON");
        maRoleMap.put(new Integer(29), "INTERNAL_FRAME");
        maRoleMap.put(new Integer(30), "LABEL");
        maRoleMap.put(new Integer(31), "LAYERED_PANE");
        maRoleMap.put(new Integer(32), "LIST");
        maRoleMap.put(new Integer(33), "LIST_ITEM");
        maRoleMap.put(new Integer(34), "MENU");
        maRoleMap.put(new Integer(35), "MENU_BAR");
        maRoleMap.put(new Integer(36), "MENU_ITEM");
        maRoleMap.put(new Integer(37), "OPTION_PANE");
        maRoleMap.put(new Integer(38), "PAGE_TAB");
        maRoleMap.put(new Integer(39), "PAGE_TAB_LIST");
        maRoleMap.put(new Integer(40), "PANEL");
        maRoleMap.put(new Integer(41), "PARAGRAPH");
        maRoleMap.put(new Integer(42), "PASSWORD_TEXT");
        maRoleMap.put(new Integer(43), "POPUP_MENU");
        maRoleMap.put(new Integer(44), "PUSH_BUTTON");
        maRoleMap.put(new Integer(45), "PROGRESS_BAR");
        maRoleMap.put(new Integer(46), "RADIO_BUTTON");
        maRoleMap.put(new Integer(47), "RADIO_MENU_ITEM");
        maRoleMap.put(new Integer(48), "ROW_HEADER");
        maRoleMap.put(new Integer(49), "ROOT_PANE");
        maRoleMap.put(new Integer(50), "SCROLL_BAR");
        maRoleMap.put(new Integer(51), "SCROLL_PANE");
        maRoleMap.put(new Integer(52), "SHAPE");
        maRoleMap.put(new Integer(53), "SEPARATOR");
        maRoleMap.put(new Integer(54), "SLIDER");
        maRoleMap.put(new Integer(55), "SPIN_BOX");
        maRoleMap.put(new Integer(56), "SPLIT_PANE");
        maRoleMap.put(new Integer(57), "STATUS_BAR");
        maRoleMap.put(new Integer(58), "TABLE");
        maRoleMap.put(new Integer(59), "TABLE_CELL");
        maRoleMap.put(new Integer(60), "TEXT");
        maRoleMap.put(new Integer(61), "TEXT_FRAME");
        maRoleMap.put(new Integer(62), "TOGGLE_BUTTON");
        maRoleMap.put(new Integer(63), "TOOL_BAR");
        maRoleMap.put(new Integer(64), "TOOL_TIP");
        maRoleMap.put(new Integer(65), "TREE");
        maRoleMap.put(new Integer(66), "VIEW_PORT");
        maRoleMap.put(new Integer(67), "WINDOW");
    }

    /**
     * Get state name for the role id.
     *
     * @param nStateId id
     *
     * @return State name
     */
    public static String getStateName(int nStateId) {
        if ((nStateId < 0) || (nStateId >= maStateNames.length)) {
            nStateId = 0;
        }

        return maStateNames[nStateId];
    }

    /**
     * Get event name for the role id.
     *
     * @param nEventId id
     *
     * @return Event name
     */
    public static String getEventName(int nEventId) {
        if ((nEventId < 0) || (nEventId >= maEventNames.length)) {
            nEventId = 0;
        }

        return maEventNames[nEventId];
    }

    /**
     * Get role name for the role id.
     *
     * @param nRole id
     *
     * @return Role name
     */
    public static String getRoleName(int nRole) {
        return (String) maRoleMap.get(new Integer(nRole));
    }

    /**
     * Get role id for the role name.
     *
     * @param sRoleName name
     *
     * @return Role id
     */
    public static int getRoleId(String sRoleName) {
        Iterator I = maRoleMap.entrySet().iterator();

        while (I.hasNext()) {
            Map.Entry aEntry = (Map.Entry) (I.next());
            String sValue = (String) aEntry.getValue();

            if (sValue.equals(sRoleName)) {
                return ((Integer) aEntry.getKey()).intValue();
            }
        }

        return -1;
    }
}
